#include "lab11-queue.h"              /* queue implementation*/

#define NUMVERTICES    8                        /* graph size */

#define true 1
#define false 0

typedef int Graph[NUMVERTICES][NUMVERTICES];   /* the type for the adj matrix*/

// Function Prototypes
void bfs(Graph G);	// Breadth First Traversal of Graph G
void bfs_aux(Graph G, int v, int visited[], Queue *Q);


/*
 * bfs() - performs a breadth first search on all the connected components of
 *         a given graph G
 */
void bfs(Graph G)
{
    int i;
    int visited[NUMVERTICES];          /* have we visited a vertex */
	Queue *Q;

	Q = InitQueue();

    for (i = 0; i < NUMVERTICES; i++)   /* initialize vertices as unvisited */
        visited[i] = false;

    for (i = 0; i < NUMVERTICES; i++)   /* if there is an unvisited component */
        if (!visited[i])                /* the search it */
            bfs_aux(G, i, visited, Q);
}

/*
 * bfs_aux() -
 *		   performs a breadth first search on G starting from vertex v.
 *         The table visited[] tags with a "1" the nodes which have been
 *		   traversed (so that we avoid visiting them again)
 */
void bfs_aux(Graph G, int v, int visited[], Queue *Q)
{
    int i;

	visited[v] = true;
	Enqueue(Q, v);

    while (!IsEmpty(Q))
    	{
			v = Dequeue(Q);
			for (i = 0; i < NUMVERTICES; i++)
		       if (G[v][i] && !visited[i]) 	// find connected vertices
		            {                       // and then add each unvisited
		                visited[i] = true;	// neighbor to the queue
		                Enqueue(Q, i);
                	}
		}
}

/*
 * Prints the Graph G on screen
 */
printgraph(Graph G)
{
	int i,j;

	for (i=0; i<NUMVERTICES; i++){
		for (j=0; j<NUMVERTICES; j++)
			printf(" %d",G[i][j]);
	 	printf("\n");
	}
}

int main()
{
 	Graph testgraph= {
		{0,1,1,1,0,0,0,0},
 		{1,0,1,0,0,0,0,0},
 		{1,1,0,1,0,0,0,0},
 		{1,0,1,0,1,1,1,0},
 		{0,0,0,1,0,1,0,0},
 		{0,0,0,1,1,0,0,0},
 		{0,0,0,1,0,0,0,1},
 		{0,0,0,0,0,0,1,0}
 	};

	printgraph(testgraph);

	printf("Breadth First Search Traversal\n");
	bfs(testgraph);
	printf("\ndone");

	return 1;
}
